#!/usr/bin/env python3

import dbus
import libvirttest
import pytest
import xmldata


class TestConnect(libvirttest.BaseTestClass):
    def test_connect_domain_create_xml(self):
        def domain_started(path, event, detail):
            if event != libvirttest.DomainEvent.STARTED:
                return
            assert detail == libvirttest.DomainEventStartedDetailType.BOOTED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_started)

        path = self.connect.DomainCreateXML(xmldata.minimal_domain_xml, 0)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    def test_connect_domain_define_xml(self):
        def domain_defined(path, event, detail):
            if event != libvirttest.DomainEvent.DEFINED:
                return
            assert detail == libvirttest.DomainEventDefinedDetailType.ADDED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_defined)

        path = self.connect.DomainDefineXML(xmldata.minimal_domain_xml)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("DomainLookupByID", 'Id'),
        ("DomainLookupByName", 'Name'),
        ("DomainLookupByUUID", 'UUID'),
    ])
    def test_connect_domain_lookup_by_id(self, lookup_method_name, lookup_item):
        """Parameterized test for all DomainLookupBy* API calls of Connect interface
        """
        original_path = self.connect.ListDomains(0)[0]
        obj, _ = self.get_test_domain()
        props = obj.GetAll('org.libvirt.Domain', dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(props[lookup_item])
        assert original_path == path

    def test_connect_find_storage_pool_sources(self):
        storageType = "logical"
        sources = self.connect.FindStoragePoolSources(storageType, "", 0)
        assert isinstance(sources, dbus.String)

    def test_connect_list_domains(self):
        domains = self.connect.ListDomains(0)
        assert isinstance(domains, dbus.Array)
        assert len(domains) == 1

        for path in domains:
            assert isinstance(path, dbus.ObjectPath)
            domain = self.bus.get_object('org.libvirt', path)

            # ensure the path exists by calling Introspect on it
            domain.Introspect(dbus_interface=dbus.INTROSPECTABLE_IFACE)

    @pytest.mark.parametrize("property_name,expected_type", [
        ("Encrypted", dbus.Boolean),
        ("Hostname", dbus.String),
        ("LibVersion", dbus.UInt64),
        ("Secure", dbus.Boolean),
        ("Version", dbus.UInt64),
    ])
    def test_connect_properties_return_type(self, property_name, expected_type):
        obj = self.bus.get_object('org.libvirt', '/org/libvirt/Test')
        props = obj.GetAll('org.libvirt.Connect', dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props[property_name], expected_type)

    def test_connect_get_sysinfo(self):
        sysinfo = self.connect.GetSysinfo(0)
        assert isinstance(sysinfo, dbus.String)

    def test_connect_interface_define_xml(self):
        path = self.connect.InterfaceDefineXML(xmldata.minimal_interface_xml, 0)
        assert isinstance(path, dbus.ObjectPath)

    @pytest.mark.usefixtures("interface_create")
    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("InterfaceLookupByName", 'Name'),
        ("InterfaceLookupByMAC", 'MAC'),
    ])
    def test_connect_interface_lookup_by_property(self, lookup_method_name, lookup_item, interface_create):
        """Parameterized test for all InterfaceLookupBy* API calls of Connect interface
        """
        original_path, _ = interface_create
        obj = self.bus.get_object('org.libvirt', original_path)
        prop = obj.Get('org.libvirt.Interface', lookup_item, dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(prop)
        assert original_path == path

    def test_connect_list_networks(self):
        networks = self.connect.ListNetworks(0)
        assert isinstance(networks, dbus.Array)
        assert len(networks) == 1

        for path in networks:
            assert isinstance(path, dbus.ObjectPath)
            network = self.bus.get_object('org.libvirt', path)

            # ensure the path exists by calling Introspect on it
            network.Introspect(dbus_interface=dbus.INTROSPECTABLE_IFACE)

    def test_connect_get_capabilities(self):
        assert isinstance(self.connect.GetCapabilities(), dbus.String)

    def test_connect_get_cpu_model_names(self):
        arch = "x86_64"
        assert isinstance(self.connect.GetCPUModelNames(arch, 0), dbus.Array)

    def test_connect_list_storage_pools(self):
        storage_pools = self.connect.ListStoragePools(0)
        assert isinstance(storage_pools, dbus.Array)
        assert len(storage_pools) == 1

        for path in storage_pools:
            assert isinstance(path, dbus.ObjectPath)
            storage_pool = self.bus.get_object('org.libvirt', path)

            # ensure the path exists by calling Introspect on it
            storage_pool.Introspect(dbus_interface=dbus.INTROSPECTABLE_IFACE)

    def test_connect_network_create_xml(self):
        def network_started(path, event):
            if event != libvirttest.NetworkEvent.STARTED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NetworkEvent', network_started)

        path = self.connect.NetworkCreateXML(xmldata.minimal_network_xml)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    def test_connect_network_define_xml(self):
        def network_defined(path, event):
            if event != libvirttest.NetworkEvent.DEFINED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NetworkEvent', network_defined)

        path = self.connect.NetworkDefineXML(xmldata.minimal_network_xml)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    @pytest.mark.usefixtures("node_device_create")
    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("NodeDeviceLookupByName", 'Name'),
    ])
    def test_connect_node_device_lookup_by_property(self, lookup_method_name, lookup_item, node_device_create):
        """Parameterized test for all NodeDeviceLookupBy* API calls of Connect interface
        """
        original_path = node_device_create
        obj = self.bus.get_object('org.libvirt', original_path)
        prop = obj.Get('org.libvirt.NodeDevice', lookup_item, dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(prop)
        assert original_path == path

    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("NetworkLookupByName", 'Name'),
        ("NetworkLookupByUUID", 'UUID'),
    ])
    def test_connect_network_lookup_by_property(self, lookup_method_name, lookup_item):
        """Parameterized test for all NetworkLookupBy* API calls of Connect interface
        """
        original_path, obj = self.get_test_network()
        prop = obj.Get('org.libvirt.Network', lookup_item, dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(prop)
        assert original_path == path

    def test_connect_node_device_create_xml(self):
        def node_device_created(path, event, _detail):
            if event != libvirttest.NodeDeviceEvent.CREATED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NodeDeviceEvent', node_device_created)

        path = self.node_device_create()
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    def test_connect_node_get_cpu_stats(self):
        stats = self.connect.NodeGetCPUStats(0, 0)
        assert isinstance(stats, dbus.Dictionary)

    def test_connect_node_get_free_memory(self):
        free_mem = self.connect.NodeGetFreeMemory()
        assert isinstance(free_mem, dbus.UInt64)

    def test_connect_node_get_cpumap(self):
        info = self.connect.NodeGetCPUMap(0)
        assert isinstance(info, dbus.Array)

    def test_connect_storage_pool_create_xml(self):
        def storage_pool_started(path, event, _detail):
            if event != libvirttest.StoragePoolEvent.STARTED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('StoragePoolEvent', storage_pool_started)

        path = self.connect.StoragePoolCreateXML(
            xmldata.minimal_storage_pool_xml, 0)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    def test_connect_storage_pool_define_xml(self):
        def storage_pool_defined(path, event, _detail):
            if event != libvirttest.StoragePoolEvent.DEFINED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('StoragePoolEvent', storage_pool_defined)

        path = self.connect.StoragePoolDefineXML(
            xmldata.minimal_storage_pool_xml, 0)
        assert isinstance(path, dbus.ObjectPath)

        self.main_loop()

    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("StoragePoolLookupByName", 'Name'),
        ("StoragePoolLookupByUUID", 'UUID'),
    ])
    def test_connect_storage_pool_lookup_by_property(self,
                                                     lookup_method_name,
                                                     lookup_item):
        """Parameterized test for all StoragePoolLookupBy* API calls of Connect interface
        """
        original_path, obj = self.get_test_storage_pool()
        prop = obj.Get('org.libvirt.StoragePool', lookup_item,
                       dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(prop)
        assert original_path == path

    @pytest.mark.usefixtures("storage_volume_create")
    @pytest.mark.parametrize("lookup_method_name,lookup_item", [
        ("StorageVolLookupByKey", 'Key'),
        ("StorageVolLookupByPath", 'Path'),
    ])
    def test_connect_storage_vol_lookup_by_property(self,
                                                    lookup_method_name,
                                                    lookup_item):
        """Parameterized test for all StorageVolLookupBy* API calls of Connect interface
        """
        original_path, obj = self.get_test_storage_volume()
        prop = obj.Get('org.libvirt.StorageVol', lookup_item,
                       dbus_interface=dbus.PROPERTIES_IFACE)
        path = getattr(self.connect, lookup_method_name)(prop)
        assert original_path == path


if __name__ == '__main__':
    libvirttest.run()



from enum import IntEnum
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GLib
import dbus
import os
import pytest
import subprocess
import sys
import time
import xmldata


root = os.environ.get('abs_top_builddir')
exe = os.path.join(root, 'src', 'libvirt-dbus')

DBusGMainLoop(set_as_default=True)


def run():
    exit(pytest.main(sys.argv))


class BaseTestClass():
    """ Base test class for whole test suite
    """
    connect = None
    bus = None
    libvirt_dbus = None
    loop = False

    @pytest.fixture(autouse=True)
    def libvirt_dbus_setup(self, request):
        """Start libvirt-dbus for each test function
        """
        os.environ['LIBVIRT_DEBUG'] = '3'
        self.libvirt_dbus = subprocess.Popen([exe, '--session'])
        self.bus = dbus.SessionBus()

        for i in range(10):
            if self.bus.name_has_owner('org.libvirt'):
                break
            time.sleep(0.1)
        else:
            raise TimeoutError('error starting libvirt-dbus')

        obj = self.bus.get_object('org.libvirt', '/org/libvirt/Test')
        self.connect = dbus.Interface(obj, 'org.libvirt.Connect')

    @pytest.fixture(autouse=True)
    def libvirt_dbus_teardown(self):
        """Terminate libvirt-dbus at the teardown of each test
        """
        yield
        self.libvirt_dbus.terminate()
        self.libvirt_dbus.wait(timeout=10)

    def main_loop(self):
        """Initializes the mainloop
        """
        assert getattr(self, 'loop', False) is False

        def timeout():
            self.loop.quit()
            del self.loop
            self.timeout = True

        self.timeout = False
        self.loop = GLib.MainLoop()
        GLib.timeout_add(2000, timeout)
        self.loop.run()
        if self.timeout:
            raise TimeoutError()

    @pytest.fixture
    def interface_create(self):
        """ Fixture to define dummy interface on the test driver

        This fixture should be used in the setup of every test manipulating
        with interfaces.
        """
        path = self.connect.InterfaceDefineXML(xmldata.minimal_interface_xml, 0)
        obj = self.bus.get_object('org.libvirt', path)
        interface_obj = dbus.Interface(obj, 'org.libvirt.Interface')
        interface_obj.Create(0)
        return path, interface_obj

    @pytest.fixture
    def node_device_create(self):
        """ Fixture to create dummy node device on the test driver

        This fixture should be used in the setup of every test manipulating
        with node devices.
        """
        # We need a usable parent nodedev: possible candidates are
        # scsi_host2 (available since libvirt 3.1.0) and
        # test-scsi-host-vport (available until libvirt 3.0.0).
        #
        # Looking up a non-existing nodedev raises an exception, so
        # we need to ignore them here: that's okay, because if we
        # can't find any then NodeDeviceCreateXML() itself will fail
        xml = xmldata.minimal_node_device_xml
        for parent in ['scsi_host2', 'test-scsi-host-vport']:
            try:
                if self.connect.NodeDeviceLookupByName(parent):
                    xml = xml.replace('@parent@', parent)
                    break
            except dbus.exceptions.DBusException:
                pass
        path = self.connect.NodeDeviceCreateXML(xml, 0)
        return path

    @pytest.fixture
    def storage_volume_create(self):
        """ Fixture to create dummy storage volume on the test driver

        This fixture should be used in the setup of every test manipulating
        with test volume.
        """
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        path = interface_obj.StorageVolCreateXML(xmldata.minimal_storage_vol_xml, 0)
        yield path

    def get_test_domain(self):
        path = self.connect.ListDomains(0)[0]
        obj = self.bus.get_object('org.libvirt', path)
        return obj, dbus.Interface(obj, 'org.libvirt.Domain')

    def get_test_network(self):
        """Fetch information for the test network from test driver

        Returns:
            (dbus.proxies.ProxyObject, dbus.proxies.ProxyObject):
            Test Network Object, Local proxy for the test Network Object.

        """
        path = self.connect.ListNetworks(0)[0]
        obj = self.bus.get_object('org.libvirt', path)
        return path, obj

    def get_test_storage_pool(self):
        """Fetch information for the test storage pool from test driver

        Returns:
            (dbus.proxies.ProxyObject, dbus.proxies.ProxyObject):
            Test StoragePool Object, Local proxy for the test StoragePool
            Object.

        """
        path = self.connect.ListStoragePools(0)[0]
        obj = self.bus.get_object('org.libvirt', path)
        return path, obj

    def get_test_storage_volume(self):
        """Fetch information for the test storage vol from test driver

        Returns:
            (dbus.proxies.ProxyObject, dbus.proxies.ProxyObject):
            Test StorageVol Object, Local proxy for the test StorageVol
            Object.

        """
        _, test_storage_pool = self.get_test_storage_pool()
        pool_iface = dbus.Interface(test_storage_pool,
                                    'org.libvirt.StoragePool')
        path = pool_iface.ListStorageVolumes(0)[0]
        obj = self.bus.get_object('org.libvirt', path)
        return path, obj


class DomainEvent(IntEnum):
    DEFINED = 0
    UNDEFINED = 1
    STARTED = 2
    SUSPENDED = 3
    RESUMED = 4
    STOPPED = 5
    SHUTDOWN = 6
    PMSUSPENDED = 7
    CRASHED = 8


class DomainEventDefinedDetailType(IntEnum):
    ADDED = 0
    UPDATED = 1
    RENAMED = 2
    FROM_SNAPSHOT = 3
    LAST = 4


class DomainEventResumedDetailType(IntEnum):
    UNPAUSED = 0
    MIGRATED = 1
    FROM_SNAPSHOT = 2
    POSTCOPY = 3
    LAST = 4


class DomainEventStartedDetailType(IntEnum):
    BOOTED = 0
    MIGRATED = 1
    RESTORED = 2
    FROM_SNAPSHOT = 3
    WAKEUP = 4
    LAST = 5


class DomainEventStoppedDetailType(IntEnum):
    SHUTDOWN = 0
    DESTROYED = 1
    CRASHED = 2
    MIGRATED = 3
    SAVED = 4
    FAILED = 5
    FROM_SNAPSHOT = 6
    LAST = 7


class DomainEventSuspendedDetailType(IntEnum):
    PAUSED = 0
    MIGRATED = 1
    IOERROR = 2
    WATCHDOG = 3
    RESTORED = 4
    FROM_SNAPSHOT = 5
    API_ERROR = 6
    POSTCOPY = 7
    POSTCOPY_FAILED = 8
    LAST = 9


class DomainEventUndefinedDetailType(IntEnum):
    REMOVED = 0
    RENAMED = 1
    LAST = 2


class DomainState(IntEnum):
    NOSTATE = 0
    RUNNING = 1
    BLOCKED = 2
    PAUSED = 3
    SHUTDOWN = 4
    SHUTOFF = 5
    CRASHED = 6
    PMSUSPENDED = 7
    LAST = 8


class NetworkEvent(IntEnum):
    DEFINED = 0
    UNDEFINED = 1
    STARTED = 2
    STOPPED = 3


class NodeDeviceEvent(IntEnum):
    CREATED = 0
    DELETED = 1


class StoragePoolBuildFlags(IntEnum):
    NEW = 0
    REPAIR = 1
    RESIZE = 2
    NO_OVERWRITE = 4
    OVERWRITE = 8


class StoragePoolEvent(IntEnum):
    DEFINED = 0
    UNDEFINED = 1
    STARTED = 2
    STOPPED = 3
    LAST = 4



#!/usr/bin/env python3

import dbus
import libvirttest
import pytest


class TestStoragePool(libvirttest.BaseTestClass):
    def test_storage_pool_autostart(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        autostart_expected = True
        interface_obj.Set('org.libvirt.StoragePool', 'Autostart',
                          autostart_expected,
                          dbus_interface=dbus.PROPERTIES_IFACE)
        autostart_current = interface_obj.Get('org.libvirt.StoragePool',
                                              'Autostart',
                                              dbus_interface=dbus.PROPERTIES_IFACE)
        assert autostart_current == dbus.Boolean(autostart_expected)

    def test_storage_pool_build(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.Destroy()
        interface_obj.Build(libvirttest.StoragePoolBuildFlags.NEW)

    def test_storage_pool_create(self):
        def storage_pool_started(path, event, _detail):
            if event != libvirttest.StoragePoolEvent.STARTED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('StoragePoolEvent', storage_pool_started)

        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.Destroy()
        interface_obj.Create(0)

        self.main_loop()

    def test_storage_pool_delete(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.Destroy()
        interface_obj.Delete(0)

    def test_storage_pool_destroy(self):
        def storage_pool_destroyed(path, event, _detail):
            if event != libvirttest.StoragePoolEvent.STOPPED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('StoragePoolEvent',
                                       storage_pool_destroyed)

        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.Destroy()

        self.main_loop()

    def test_storage_pool_get_info(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        info = interface_obj.GetInfo()
        assert isinstance(info, dbus.Struct)

    def test_storage_pool_get_xml_description(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        info = interface_obj.GetXMLDesc(0)
        assert isinstance(info, dbus.String)

    def test_storage_pool_list_storage_volumes(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        storage_vols = interface_obj.ListStorageVolumes(0)
        assert isinstance(storage_vols, dbus.Array)
        assert len(storage_vols) == 0

    def test_storage_pool_properties_type(self):
        _, obj = self.get_test_storage_pool()

        props = obj.GetAll('org.libvirt.StoragePool',
                           dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Active'], dbus.Boolean)
        assert isinstance(props['Autostart'], dbus.Boolean)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['Persistent'], dbus.Boolean)
        assert isinstance(props['UUID'], dbus.String)

    def test_storage_pool_undefine(self):
        def storage_pool_undefined(path, event, _detail):
            if event != libvirttest.StoragePoolEvent.UNDEFINED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('StoragePoolEvent',
                                       storage_pool_undefined)

        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.Destroy()
        interface_obj.Undefine()

        self.main_loop()

    def test_storage_pool_refresh(self):
        _, test_storage_pool = self.get_test_storage_pool()
        interface_obj = dbus.Interface(test_storage_pool,
                                       'org.libvirt.StoragePool')
        interface_obj.connect_to_signal('Refresh',
                                        lambda: self.loop.quit())
        interface_obj.Refresh(0)

        self.main_loop()

    def test_storage_pool_volume_create(self, storage_volume_create):
        assert isinstance(storage_volume_create, dbus.ObjectPath)

    @pytest.mark.usefixtures('storage_volume_create')
    def test_storage_pool_volume_create_xml_from(self):
        minimal_storage_vol_clone_xml = '''
        <volume>
          <name>clone.img</name>
          <capacity unit="G">1</capacity>
        </volume>
        '''
        _, test_storage_vol = self.get_test_storage_volume()
        props = test_storage_vol.GetAll('org.libvirt.StorageVol',
                                        dbus_interface=dbus.PROPERTIES_IFACE)
        test_storage_vol_key = str(props['Key'])

        _, test_storage_pool = self.get_test_storage_pool()
        storage_pool_iface = dbus.Interface(test_storage_pool,
                                            'org.libvirt.StoragePool')

        new_vol_path = storage_pool_iface.StorageVolCreateXMLFrom(minimal_storage_vol_clone_xml,
                                                                  test_storage_vol_key,
                                                                  0)
        assert isinstance(new_vol_path, dbus.ObjectPath)


@pytest.mark.usefixtures('storage_volume_create')
class TestStorageVolume(libvirttest.BaseTestClass):
    def test_storage_vol_delete(self):
        test_storage_vol_path, test_storage_vol = self.get_test_storage_volume()
        interface_obj = dbus.Interface(test_storage_vol,
                                       'org.libvirt.StorageVol')
        interface_obj.Delete(0)

    def test_storage_vol_properties_type(self):
        _, obj = self.get_test_storage_volume()

        props = obj.GetAll('org.libvirt.StorageVol',
                           dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Key'], dbus.String)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['Path'], dbus.String)

    def test_storage_vol_get_xml_description(self):
        _, test_storage_vol = self.get_test_storage_volume()
        interface_obj = dbus.Interface(test_storage_vol,
                                       'org.libvirt.StorageVol')
        xml = interface_obj.GetXMLDesc(0)
        assert isinstance(xml, dbus.String)


if __name__ == '__main__':
    libvirttest.run()



import os
import pytest
import subprocess


@pytest.fixture(scope="session", autouse=True)
def dbus_daemon_session():
    """Fixture to start D-Bus message bus session daemon for use in the whole test suite.
    """
    dbus_daemon = subprocess.Popen(['dbus-daemon', '--session', '--print-address'],
                                   stdout=subprocess.PIPE, universal_newlines=True)
    os.environ['DBUS_SESSION_BUS_ADDRESS'] = dbus_daemon.stdout.readline().strip()
    yield
    dbus_daemon.terminate()
    dbus_daemon.wait(timeout=10)



#!/usr/bin/env python3

import dbus
import libvirttest
import pytest


@pytest.mark.usefixtures("interface_create")
class TestInterface(libvirttest.BaseTestClass):
    """ Tests for methods and properties of the Interface interface
    """

    def test_interface_undefine(self, interface_create):
        _, interface_obj = interface_create
        interface_obj.Destroy(0)
        interface_obj.Undefine()

    def test_interface_destroy(self, interface_create):
        _, interface_obj = interface_create
        interface_obj.Destroy(0)

    def test_interface_create(self, interface_create):
        _, interface_obj = interface_create
        interface_obj.Destroy(0)
        interface_obj.Create(0)

    def test_interface_get_xml_description(self, interface_create):
        _, interface_obj = interface_create
        assert isinstance(interface_obj.GetXMLDesc(0), dbus.String)

    def test_interface_properties_type(self, interface_create):
        """ Ensure correct return type for Interface properties
        """
        test_interface_path, _ = interface_create
        obj = self.bus.get_object('org.libvirt', test_interface_path)
        props = obj.GetAll('org.libvirt.Interface', dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['MAC'], dbus.String)
        assert isinstance(props['Active'], dbus.Boolean)


if __name__ == '__main__':
    libvirttest.run()



# -*- coding: utf-8 -*-
"""Minimal libvirt XML definition of entities to be used in the test suite.
"""

minimal_domain_xml = '''
<domain type="test">
  <name>foo</name>
  <memory>1024</memory>
  <os>
    <type>hvm</type>
  </os>
</domain>
'''

minimal_interface_xml = '''
<interface type='ethernet' name='test-iface'>
  <start mode='onboot'/>
  <mac address='11:22:33:44:55:66'/>
  <mtu size='1492'/>
  <protocol family='ipv4'>
    <ip address='192.168.15.5' prefix='24'/>
    <route gateway='192.168.15.1'/>
  </protocol>
</interface>
'''

minimal_network_xml = '''
<network>
  <name>bar</name>
  <uuid>004b96e12d78c30f5aa5f03c87d21e69</uuid>
  <bridge name='brdefault'/>
  <forward dev='eth0'/>
  <ip address='192.168.122.1' netmask='255.255.255.0'>
    <dhcp>
      <range start='192.168.122.128' end='192.168.122.253'/>
    </dhcp>
  </ip>
</network>
'''

minimal_node_device_xml = '''
<device>
  <name>scsi_host22</name>
  <parent>@parent@</parent>
  <capability type='scsi_host'>
    <host>22</host>
    <unique_id>22</unique_id>
    <capability type='fc_host'>
      <wwnn>2000000098765432</wwnn>
      <wwpn>1000000098765432</wwpn>
      <fabric_wwn>2000000098769876</fabric_wwn>
    </capability>
  </capability>
</device>
'''

minimal_storage_pool_xml = '''
<pool type='dir'>
  <name>foo</name>
  <uuid>35bb2ad9-388a-cdfe-461a-b8907f6e53fe</uuid>
  <target>
    <path>/foo</path>
  </target>
</pool>
'''

minimal_storage_vol_xml = '''
<volume>
  <name>sparse.img</name>
  <capacity unit="G">2</capacity>
  <target>
    <path>/var/lib/virt/images/sparse.img</path>
  </target>
</volume>
'''



#!/usr/bin/env python3

import dbus
import libvirttest

DBUS_EXCEPTION_MISSING_FUNCTION = 'this function is not supported by the connection driver'


class TestDomain(libvirttest.BaseTestClass):
    def test_api(self):
        obj, domain = self.get_test_domain()

        props = obj.GetAll('org.libvirt.Domain', dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Active'], dbus.Boolean)
        assert isinstance(props['Autostart'], dbus.Boolean)
        assert isinstance(props['Id'], dbus.UInt32)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['OSType'], dbus.String)
        assert isinstance(props['Persistent'], dbus.Boolean)
        assert any([isinstance(props['SchedulerType'], dbus.Struct),
                    isinstance(props['SchedulerType'][0], dbus.String),
                    isinstance(props['SchedulerType'][1], dbus.Int32)])
        assert isinstance(props['Updated'], dbus.Boolean)
        assert isinstance(props['UUID'], dbus.String)

        # Call all methods except Reset and GetStats, because the test backend
        # doesn't support those

        xml = domain.GetXMLDesc(0)
        assert isinstance(xml, dbus.String)

        domain.Reboot(0)
        domain.Shutdown(0)
        domain.Create(0)
        try:
            domain.Destroy(0)
        except dbus.exceptions.DBusException as e:
            if not any(DBUS_EXCEPTION_MISSING_FUNCTION in arg for arg in e.args):
                raise e
        domain.Undefine(0)

    def test_domain_autostart(self):
        _, domain = self.get_test_domain()
        autostart_expected = True
        domain.Set('org.libvirt.Domain', 'Autostart', autostart_expected, dbus_interface=dbus.PROPERTIES_IFACE)
        autostart_current = domain.Get('org.libvirt.Domain', 'Autostart', dbus_interface=dbus.PROPERTIES_IFACE)
        assert autostart_current == dbus.Boolean(autostart_expected)

    def test_domain_managed_save(self):
        def domain_stopped(path, event, detail):
            if event != libvirttest.DomainEvent.STOPPED:
                return
            assert detail == libvirttest.DomainEventStoppedDetailType.SAVED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_stopped)

        obj, domain = self.get_test_domain()
        domain.ManagedSave(0)
        assert domain.HasManagedSaveImage(0) == dbus.Boolean(True)
        state, _ = domain.GetState(0)
        assert state == libvirttest.DomainState.SHUTOFF
        domain.ManagedSaveRemove(0)
        assert domain.HasManagedSaveImage(0) == dbus.Boolean(False)

        self.main_loop()

    def test_domain_metadata(self):
        metadata_description = 0
        obj, domain = self.get_test_domain()
        description_expected = "This is the Test domain"
        domain.SetMetadata(metadata_description,
                           description_expected, "", "", 0)
        assert description_expected == domain.GetMetadata(metadata_description, "", 0)

    def test_resume(self):
        def domain_resumed(path, event, detail):
            if event != libvirttest.DomainEvent.RESUMED:
                return
            assert detail == libvirttest.DomainEventResumedDetailType.UNPAUSED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_resumed)

        obj, domain = self.get_test_domain()
        domain.Suspend()
        domain.Resume()

        state, _ = domain.GetState(0)
        assert state == libvirttest.DomainState.RUNNING

        self.main_loop()

    def test_shutdown(self):
        def domain_stopped(path, event, detail):
            if event != libvirttest.DomainEvent.STOPPED:
                return
            assert detail == libvirttest.DomainEventStoppedDetailType.SHUTDOWN
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_stopped)

        obj, domain = self.get_test_domain()
        domain.Shutdown(0)

        state, _ = domain.GetState(0)
        assert state == libvirttest.DomainState.SHUTOFF

        self.main_loop()

    def test_suspend(self):
        def domain_suspended(path, event, detail):
            if event != libvirttest.DomainEvent.SUSPENDED:
                return
            assert detail == libvirttest.DomainEventSuspendedDetailType.PAUSED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_suspended)

        obj, domain = self.get_test_domain()
        domain.Suspend()

        state, _ = domain.GetState(0)
        assert state == libvirttest.DomainState.PAUSED

        self.main_loop()

    def test_undefine(self):
        def domain_undefined(path, event, detail):
            if event != libvirttest.DomainEvent.UNDEFINED:
                return
            assert detail == libvirttest.DomainEventUndefinedDetailType.REMOVED
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('DomainEvent', domain_undefined)

        _, domain = self.get_test_domain()
        domain.Shutdown(0)
        domain.Undefine(0)

        self.main_loop()

    def test_domain_vcpus(self):
        obj, domain = self.get_test_domain()
        vcpus_expected = 2
        domain.SetVcpus(vcpus_expected, 0)
        assert domain.GetVcpus(0) == dbus.Int32(vcpus_expected)

    def test_domain_vcpu_pin_info(self):
        obj, domain = self.get_test_domain()
        pinInfo_expected = [
            [True, True, True, True, True, True, True, True],
            [True, True, True, True, True, True, True, True]
        ]
        pinInfo = domain.GetVcpuPinInfo(0)
        assert pinInfo == pinInfo_expected


if __name__ == '__main__':
    libvirttest.run()



#!/usr/bin/env python3

import dbus
import libvirttest


class TestNodeDevice(libvirttest.BaseTestClass):
    """ Tests for methods and properties of the NodeDevice interface
    """

    def test_node_device_destroy(self, node_device_create):
        def node_device_deleted(path, event, _detail):
            if event != libvirttest.NodeDeviceEvent.DELETED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NodeDeviceEvent', node_device_deleted)

        test_node_device_path = node_device_create
        obj = self.bus.get_object('org.libvirt', test_node_device_path)
        interface_obj = dbus.Interface(obj, 'org.libvirt.NodeDevice')
        interface_obj.Destroy()

        self.main_loop()

    def test_node_device_get_xml_description(self, node_device_create):
        test_node_device_path = node_device_create
        obj = self.bus.get_object('org.libvirt', test_node_device_path)
        interface_obj = dbus.Interface(obj, 'org.libvirt.NodeDevice')
        assert isinstance(interface_obj.GetXMLDesc(0), dbus.String)

    def test_node_device_list_caps(self, node_device_create):
        test_node_device_path = node_device_create
        obj = self.bus.get_object('org.libvirt', test_node_device_path)
        interface_obj = dbus.Interface(obj, 'org.libvirt.NodeDevice')
        assert isinstance(interface_obj.ListCaps(), dbus.Array)

    def test_node_device_properties_type(self, node_device_create):
        """ Ensure correct return type for NodeDevice properties
        """
        test_node_device_path = node_device_create
        obj = self.bus.get_object('org.libvirt', test_node_device_path)
        props = obj.GetAll('org.libvirt.NodeDevice', dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['Parent'], dbus.String)


if __name__ == '__main__':
    libvirttest.run()



#!/usr/bin/env python3

import dbus
import libvirttest
import pytest


class TestNetwork(libvirttest.BaseTestClass):
    """ Tests for methods and properties of the Network interface
    """

    ip_dhcp_host_xml = '''
    <host mac='00:16:3e:77:e2:ed' name='foo.example.com' ip='192.168.122.10'/>
    '''

    def test_network_properties_type(self):
        """ Ensure correct return type for Network properties
        """
        _, obj = self.get_test_network()
        props = obj.GetAll('org.libvirt.Network', dbus_interface=dbus.PROPERTIES_IFACE)
        assert isinstance(props['Active'], dbus.Boolean)
        assert isinstance(props['Autostart'], dbus.Boolean)
        assert isinstance(props['Name'], dbus.String)
        assert isinstance(props['Persistent'], dbus.Boolean)
        assert isinstance(props['UUID'], dbus.String)

    def test_network_autostart(self):
        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        autostart_expected = True
        interface_obj.Set('org.libvirt.Network', 'Autostart', autostart_expected, dbus_interface=dbus.PROPERTIES_IFACE)
        autostart_current = interface_obj.Get('org.libvirt.Network', 'Autostart', dbus_interface=dbus.PROPERTIES_IFACE)
        assert autostart_current == dbus.Boolean(autostart_expected)

    def test_network_create(self):
        def domain_started(path, event):
            if event != libvirttest.NetworkEvent.STARTED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NetworkEvent', domain_started)

        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        interface_obj.Destroy()
        interface_obj.Create()

        self.main_loop()

    def test_network_destroy(self):
        def network_stopped(path, event):
            if event != libvirttest.NetworkEvent.STOPPED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NetworkEvent', network_stopped)

        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        interface_obj.Destroy()

        self.main_loop()

    def test_network_get_xml_description(self):
        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        assert isinstance(interface_obj.GetXMLDesc(0), dbus.String)

    def test_network_undefine(self):
        def domain_undefined(path, event):
            if event != libvirttest.NetworkEvent.UNDEFINED:
                return
            assert isinstance(path, dbus.ObjectPath)
            self.loop.quit()

        self.connect.connect_to_signal('NetworkEvent', domain_undefined)

        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        interface_obj.Destroy()
        interface_obj.Undefine()

        self.main_loop()

    @pytest.mark.parametrize("command, section, parentIndex, xml_str, flags", [
        ('4', '4', 0, ip_dhcp_host_xml, 0),  # add-first, ip-dhcp-host
    ])
    def test_network_update(self, command, section, parentIndex, xml_str, flags):
        _, test_network = self.get_test_network()
        interface_obj = dbus.Interface(test_network, 'org.libvirt.Network')
        interface_obj.Update(command, section, parentIndex, xml_str, flags)
        updated_netxml = interface_obj.GetXMLDesc(0)
        assert (xml_str.strip() in updated_netxml)


if __name__ == '__main__':
    libvirttest.run()
